@@ -1,53 +1,17 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qmimedata.h"
 
 #include "private/qobject_p.h"
 #include "qurl.h"
 #include "qstringlist.h"
-#if QT_CONFIG(textcodec)
-#include "qtextcodec.h"
-#endif
+#include "qstringconverter.h"
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 static inline QString textUriListLiteral() { return QStringLiteral("text/uri-list"); }
 static inline QString textHtmlLiteral() { return QStringLiteral("text/html"); }
 static inline QString textPlainLiteral() { return QStringLiteral("text/plain"); }
@@ -60,7 +24,7 @@ struct QMimeDataStruct
     QString format;
     QVariant data;
 };
-Q_DECLARE_TYPEINFO(QMimeDataStruct, Q_MOVABLE_TYPE);
+Q_DECLARE_TYPEINFO(QMimeDataStruct, Q_RELOCATABLE_TYPE);
 
 class QMimeDataPrivate : public QObjectPrivate
 {
@@ -70,7 +34,7 @@ public:
     void setData(const QString &format, const QVariant &data);
     QVariant getData(const QString &format) const;
 
-    QVariant retrieveTypedData(const QString &format, QMetaType::Type type) const;
+    QVariant retrieveTypedData(const QString &format, QMetaType type) const;
 
     std::vector<QMimeDataStruct>::iterator find(const QString &format) noexcept {
         const auto formatEquals = [](const QString &format) {
@@ -112,24 +76,25 @@ QVariant QMimeDataPrivate::getData(const QString &format) const
         return it->data;
 }
 
-QVariant QMimeDataPrivate::retrieveTypedData(const QString &format, QMetaType::Type type) const
+QVariant QMimeDataPrivate::retrieveTypedData(const QString &format, QMetaType type) const
 {
     Q_Q(const QMimeData);
+    int typeId = type.id();
 
-    QVariant data = q->retrieveData(format, QVariant::Type(type));
+    QVariant data = q->retrieveData(format, type);
 
     // Text data requested: fallback to URL data if available
-    if (format == QLatin1String("text/plain") && !data.isValid()) {
-        data = retrieveTypedData(textUriListLiteral(), QMetaType::QVariantList);
-        if (data.userType() == QMetaType::QUrl) {
+    if (format == "text/plain"_L1 && !data.isValid()) {
+        data = retrieveTypedData(textUriListLiteral(), QMetaType(QMetaType::QVariantList));
+        if (data.metaType().id() == QMetaType::QUrl) {
             data = QVariant(data.toUrl().toDisplayString());
-        } else if (data.userType() == QMetaType::QVariantList) {
+        } else if (data.metaType().id() == QMetaType::QVariantList) {
             QString text;
             int numUrls = 0;
             const QList<QVariant> list = data.toList();
             for (int i = 0; i < list.size(); ++i) {
-                if (list.at(i).userType() == QMetaType::QUrl) {
-                    text += list.at(i).toUrl().toDisplayString() + QLatin1Char('\n');
+                if (list.at(i).metaType().id() == QMetaType::QUrl) {
+                    text += list.at(i).toUrl().toDisplayString() + u'\n';
                     ++numUrls;
                 }
             }
@@ -139,40 +104,44 @@ QVariant QMimeDataPrivate::retrieveTypedData(const QString &format, QMetaType::T
         }
     }
 
-    if (data.userType() == type || !data.isValid())
+    if (data.metaType() == type || !data.isValid())
         return data;
 
-    // provide more conversion possiblities than just what QVariant provides
+    // provide more conversion possibilities than just what QVariant provides
 
     // URLs can be lists as well...
-    if ((type == QMetaType::QUrl && data.userType() == QMetaType::QVariantList)
-        || (type == QMetaType::QVariantList && data.userType() == QMetaType::QUrl))
+    if ((typeId == QMetaType::QUrl && data.metaType().id() == QMetaType::QVariantList)
+        || (typeId == QMetaType::QVariantList && data.metaType().id() == QMetaType::QUrl))
         return data;
 
     // images and pixmaps are interchangeable
-    if ((type == QMetaType::QPixmap && data.userType() == QMetaType::QImage)
-        || (type == QMetaType::QImage && data.userType() == QMetaType::QPixmap))
+    if ((typeId == QMetaType::QPixmap && data.metaType().id() == QMetaType::QImage)
+        || (typeId == QMetaType::QImage && data.metaType().id() == QMetaType::QPixmap))
         return data;
 
-    if (data.userType() == QMetaType::QByteArray) {
+    if (data.metaType().id() == QMetaType::QByteArray) {
         // see if we can convert to the requested type
-        switch(type) {
-#if QT_CONFIG(textcodec)
+        switch (typeId) {
         case QMetaType::QString: {
             const QByteArray ba = data.toByteArray();
-            QTextCodec *codec = QTextCodec::codecForName("utf-8");
-            if (format == QLatin1String("text/html"))
-                codec = QTextCodec::codecForHtml(ba, codec);
-            return codec->toUnicode(ba);
+            if (ba.isNull())
+                return QVariant();
+            if (format == "text/html"_L1) {
+                QStringDecoder decoder = QStringDecoder::decoderForHtml(ba);
+                if (decoder.isValid()) {
+                    return QString(decoder(ba));
+                }
+                // fall back to utf8
+            }
+            return QString::fromUtf8(ba);
         }
-#endif // textcodec
         case QMetaType::QColor: {
             QVariant newData = data;
-            newData.convert(QMetaType::QColor);
+            newData.convert(QMetaType(QMetaType::QColor));
             return newData;
         }
         case QMetaType::QVariantList: {
-            if (format != QLatin1String("text/uri-list"))
+            if (format != "text/uri-list"_L1)
                 break;
             Q_FALLTHROUGH();
         }
@@ -197,10 +166,10 @@ QVariant QMimeDataPrivate::retrieveTypedData(const QString &format, QMetaType::T
             break;
         }
 
-    } else if (type == QMetaType::QByteArray) {
+    } else if (typeId == QMetaType::QByteArray) {
 
         // try to convert to bytearray
-        switch (data.userType()) {
+        switch (data.metaType().id()) {
         case QMetaType::QByteArray:
         case QMetaType::QColor:
             return data.toByteArray();
@@ -213,7 +182,7 @@ QVariant QMimeDataPrivate::retrieveTypedData(const QString &format, QMetaType::T
             QByteArray result;
             QList<QVariant> list = data.toList();
             for (int i = 0; i < list.size(); ++i) {
-                if (list.at(i).userType() == QMetaType::QUrl) {
+                if (list.at(i).metaType().id() == QMetaType::QUrl) {
                     result += list.at(i).toUrl().toEncoded();
                     result += "\r\n";
                 }
@@ -311,12 +280,12 @@ QVariant QMimeDataPrivate::retrieveTypedData(const QString &format, QMetaType::T
     \snippet code/src_corelib_kernel_qmimedata.cpp 8
 
     On Windows, the MIME format does not always map directly to the
-    clipboard formats. Qt provides QWinMime to map clipboard
+    clipboard formats. Qt provides QWindowsMimeConverter to map clipboard
     formats to open-standard MIME formats. Similarly, the
-    QMacPasteboardMime maps MIME to Mac flavors.
+    QUtiMimeConverter maps MIME to Uniform Type Identifiers on macOS and iOS.
 
     \sa QClipboard, QDragEnterEvent, QDragMoveEvent, QDropEvent, QDrag,
-        QMacPasteboardMime, {Drag and Drop}
+        {Drag and Drop}
 */
 
 /*!
@@ -344,14 +313,14 @@ QMimeData::~QMimeData()
 QList<QUrl> QMimeData::urls() const
 {
     Q_D(const QMimeData);
-    QVariant data = d->retrieveTypedData(textUriListLiteral(), QMetaType::QVariantList);
+    QVariant data = d->retrieveTypedData(textUriListLiteral(), QMetaType(QMetaType::QVariantList));
     QList<QUrl> urls;
-    if (data.userType() == QMetaType::QUrl)
+    if (data.metaType().id() == QMetaType::QUrl)
         urls.append(data.toUrl());
-    else if (data.userType() == QMetaType::QVariantList) {
+    else if (data.metaType().id() == QMetaType::QVariantList) {
         QList<QVariant> list = data.toList();
         for (int i = 0; i < list.size(); ++i) {
-            if (list.at(i).userType() == QMetaType::QUrl)
+            if (list.at(i).metaType().id() == QMetaType::QUrl)
                 urls.append(list.at(i).toUrl());
         }
     }
@@ -404,11 +373,11 @@ bool QMimeData::hasUrls() const
 QString QMimeData::text() const
 {
     Q_D(const QMimeData);
-    QVariant utf8Text = d->retrieveTypedData(textPlainUtf8Literal(), QMetaType::QString);
+    QVariant utf8Text = d->retrieveTypedData(textPlainUtf8Literal(), QMetaType(QMetaType::QString));
     if (!utf8Text.isNull())
         return utf8Text.toString();
 
-    QVariant data = d->retrieveTypedData(textPlainLiteral(), QMetaType::QString);
+    QVariant data = d->retrieveTypedData(textPlainLiteral(), QMetaType(QMetaType::QString));
     return data.toString();
 }
 
@@ -444,7 +413,7 @@ bool QMimeData::hasText() const
 QString QMimeData::html() const
 {
     Q_D(const QMimeData);
-    QVariant data = d->retrieveTypedData(textHtmlLiteral(), QMetaType::QString);
+    QVariant data = d->retrieveTypedData(textHtmlLiteral(), QMetaType(QMetaType::QString));
     return data.toString();
 }
 
@@ -486,7 +455,7 @@ bool QMimeData::hasHtml() const
 QVariant QMimeData::imageData() const
 {
     Q_D(const QMimeData);
-    return d->retrieveTypedData(applicationXQtImageLiteral(), QMetaType::QImage);
+    return d->retrieveTypedData(applicationXQtImageLiteral(), QMetaType(QMetaType::QImage));
 }
 
 /*!
@@ -533,7 +502,7 @@ bool QMimeData::hasImage() const
 QVariant QMimeData::colorData() const
 {
     Q_D(const QMimeData);
-    return d->retrieveTypedData(applicationXColorLiteral(), QMetaType::QColor);
+    return d->retrieveTypedData(applicationXColorLiteral(), QMetaType(QMetaType::QColor));
 }
 
 /*!
@@ -568,7 +537,7 @@ bool QMimeData::hasColor() const
 QByteArray QMimeData::data(const QString &mimeType) const
 {
     Q_D(const QMimeData);
-    QVariant data = d->retrieveTypedData(mimeType, QMetaType::QByteArray);
+    QVariant data = d->retrieveTypedData(mimeType, QMetaType(QMetaType::QByteArray));
     return data.toByteArray();
 }
 
@@ -582,17 +551,15 @@ QByteArray QMimeData::data(const QString &mimeType) const
 
     Note that if you want to use a custom data type in an item view drag and drop
     operation, you must register it as a Qt \l{QMetaType}{meta type}, using the
-    Q_DECLARE_METATYPE() macro, and implement stream operators for it. The stream
-    operators must then be registered with the qRegisterMetaTypeStreamOperators()
-    function.
+    Q_DECLARE_METATYPE() macro, and implement stream operators for it.
 
-    \sa hasFormat(), QMetaType, {QMetaType::}{qRegisterMetaTypeStreamOperators()}
+    \sa hasFormat(), QMetaType, {QMetaType::}{Q_DECLARE_METATYPE()}
 */
 void QMimeData::setData(const QString &mimeType, const QByteArray &data)
 {
     Q_D(QMimeData);
 
-    if (mimeType == QLatin1String("text/uri-list")) {
+    if (mimeType == "text/uri-list"_L1) {
         QByteArray ba = data;
         if (ba.endsWith('\0'))
             ba.chop(1);
@@ -660,7 +627,7 @@ QStringList QMimeData::formats() const
 
     \sa data()
 */
-QVariant QMimeData::retrieveData(const QString &mimeType, QVariant::Type type) const
+QVariant QMimeData::retrieveData(const QString &mimeType, QMetaType type) const
 {
     Q_UNUSED(type);
     Q_D(const QMimeData);
