@@ -1,57 +1,24 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qthreadpool.h"
 #include "qthreadpool_p.h"
 #include "qdeadlinetimer.h"
+#include "qcoreapplication.h"
 
 #include <algorithm>
+#include <memory>
 
 QT_BEGIN_NAMESPACE
 
-Q_GLOBAL_STATIC(QThreadPool, theInstance)
+using namespace Qt::StringLiterals;
 
 /*
     QThread wrapper, provides synchronization against a ThreadPool
 */
 class QThreadPoolThread : public QThread
 {
+    Q_OBJECT
 public:
     QThreadPoolThread(QThreadPoolPrivate *manager);
     void run() override;
@@ -88,8 +55,8 @@ void QThreadPoolThread::run()
 
         do {
             if (r) {
-                const bool autoDelete = r->autoDelete();
-
+                // If autoDelete() is false, r might already be deleted after run(), so check status now.
+                const bool del = r->autoDelete();
 
                 // run the task
                 locker.unlock();
@@ -106,20 +73,19 @@ void QThreadPoolThread::run()
                     throw;
                 }
 #endif
-                locker.relock();
 
-                if (autoDelete && !--r->ref)
+                if (del)
                     delete r;
+                locker.relock();
             }
 
-            // if too many threads are active, expire this thread
+            // if too many threads are active, stop working in this one
             if (manager->tooManyThreadsActive())
                 break;
 
-            if (manager->queue.isEmpty()) {
-                r = nullptr;
+            // all work is done, time to wait for more
+            if (manager->queue.isEmpty())
                 break;
-            }
 
             QueuePage *page = manager->queue.first();
             r = page->pop();
@@ -130,26 +96,32 @@ void QThreadPoolThread::run()
             }
         } while (true);
 
-        // if too many threads are active, expire this thread
-        bool expired = manager->tooManyThreadsActive();
-        if (!expired) {
-            manager->waitingThreads.enqueue(this);
+        // this thread is about to be deleted, do not wait or expire
+        if (!manager->allThreads.contains(this)) {
             registerThreadInactive();
-            // wait for work, exiting after the expiry timeout is reached
-            runnableReady.wait(locker.mutex(), QDeadlineTimer(manager->expiryTimeout));
-            ++manager->activeThreads;
-            if (manager->waitingThreads.removeOne(this))
-                expired = true;
-            if (!manager->allThreads.contains(this)) {
-                registerThreadInactive();
-                break;
-            }
+            return;
         }
-        if (expired) {
+
+        // if too many threads are active, expire this thread
+        if (manager->tooManyThreadsActive()) {
             manager->expiredThreads.enqueue(this);
             registerThreadInactive();
-            break;
+            return;
         }
+        manager->waitingThreads.enqueue(this);
+        registerThreadInactive();
+        // wait for work, exiting after the expiry timeout is reached
+        runnableReady.wait(locker.mutex(), QDeadlineTimer(manager->expiryTimeout));
+        // this thread is about to be deleted, do not work or expire
+        if (!manager->allThreads.contains(this)) {
+            Q_ASSERT(manager->queue.isEmpty());
+            return;
+        }
+        if (manager->waitingThreads.removeOne(this)) {
+            manager->expiredThreads.enqueue(this);
+            return;
+        }
+        ++manager->activeThreads;
     }
 }
 
@@ -176,10 +148,10 @@ bool QThreadPoolPrivate::tryStart(QRunnable *task)
     }
 
     // can't do anything if we're over the limit
-    if (activeThreadCount() >= maxThreadCount)
+    if (areAllThreadsActive())
         return false;
 
-    if (waitingThreads.count() > 0) {
+    if (!waitingThreads.isEmpty()) {
         // recycle an available thread
         enqueueTask(task);
         waitingThreads.takeFirst()->runnableReady.wakeOne();
@@ -193,10 +165,13 @@ bool QThreadPoolPrivate::tryStart(QRunnable *task)
 
         ++activeThreads;
 
-        if (task->autoDelete())
-            ++task->ref;
         thread->runnable = task;
-        thread->start();
+
+        // Ensure that the thread has actually finished, otherwise the following
+        // start() has no effect.
+        thread->wait();
+        Q_ASSERT(thread->isFinished());
+        thread->start(threadPriority);
         return true;
     }
 
@@ -213,10 +188,7 @@ inline bool comparePriority(int priority, const QueuePage *p)
 void QThreadPoolPrivate::enqueueTask(QRunnable *runnable, int priority)
 {
     Q_ASSERT(runnable != nullptr);
-    if (runnable->autoDelete())
-        ++runnable->ref;
-
-    for (QueuePage *page : qAsConst(queue)) {
+    for (QueuePage *page : std::as_const(queue)) {
         if (page->priority() == priority && !page->isFull()) {
             page->push(runnable);
             return;
@@ -228,9 +200,9 @@ void QThreadPoolPrivate::enqueueTask(QRunnable *runnable, int priority)
 
 int QThreadPoolPrivate::activeThreadCount() const
 {
-    return (allThreads.count()
-            - expiredThreads.count()
-            - waitingThreads.count()
+    return (allThreads.size()
+            - expiredThreads.size()
+            - waitingThreads.size()
             + reservedThreads);
 }
 
@@ -251,10 +223,16 @@ void QThreadPoolPrivate::tryToStartMoreThreads()
     }
 }
 
+bool QThreadPoolPrivate::areAllThreadsActive() const
+{
+    const int activeThreadCount = this->activeThreadCount();
+    return activeThreadCount >= maxThreadCount() && (activeThreadCount - reservedThreads) >= 1;
+}
+
 bool QThreadPoolPrivate::tooManyThreadsActive() const
 {
     const int activeThreadCount = this->activeThreadCount();
-    return activeThreadCount > maxThreadCount && (activeThreadCount - reservedThreads) > 1;
+    return activeThreadCount > maxThreadCount() && (activeThreadCount - reservedThreads) > 1;
 }
 
 /*!
@@ -263,34 +241,36 @@ bool QThreadPoolPrivate::tooManyThreadsActive() const
 void QThreadPoolPrivate::startThread(QRunnable *runnable)
 {
     Q_ASSERT(runnable != nullptr);
-    QScopedPointer <QThreadPoolThread> thread(new QThreadPoolThread(this));
-    thread->setObjectName(QLatin1String("Thread (pooled)"));
-    Q_ASSERT(!allThreads.contains(thread.data())); // if this assert hits, we have an ABA problem (deleted threads don't get removed here)
-    allThreads.insert(thread.data());
+    auto thread = std::make_unique<QThreadPoolThread>(this);
+    if (objectName.isEmpty())
+        objectName = u"Thread (pooled)"_s;
+    thread->setObjectName(objectName);
+    Q_ASSERT(!allThreads.contains(thread.get())); // if this assert hits, we have an ABA problem (deleted threads don't get removed here)
+    allThreads.insert(thread.get());
     ++activeThreads;
 
-    if (runnable->autoDelete())
-        ++runnable->ref;
     thread->runnable = runnable;
-    thread.take()->start();
+    thread.release()->start(threadPriority);
 }
 
 /*!
     \internal
 
     Helper function only to be called from waitForDone(int)
+
+    Deletes all current threads.
 */
 void QThreadPoolPrivate::reset()
 {
     // move the contents of the set out so that we can iterate without the lock
-    QSet<QThreadPoolThread *> allThreadsCopy;
-    allThreadsCopy.swap(allThreads);
+    auto allThreadsCopy = std::exchange(allThreads, {});
     expiredThreads.clear();
     waitingThreads.clear();
+
     mutex.unlock();
 
-    for (QThreadPoolThread *thread: qAsConst(allThreadsCopy)) {
-        if (!thread->isFinished()) {
+    for (QThreadPoolThread *thread : std::as_const(allThreadsCopy)) {
+        if (thread->isRunning()) {
             thread->runnableReady.wakeAll();
             thread->wait();
         }
@@ -317,29 +297,30 @@ bool QThreadPoolPrivate::waitForDone(int msecs)
 {
     QMutexLocker locker(&mutex);
     QDeadlineTimer timer(msecs);
-    do {
-        if (!waitForDone(timer))
-            return false;
-        reset();
-        // More threads can be started during reset(), in that case continue
-        // waiting if we still have time left.
-    } while ((!queue.isEmpty() || activeThreads) && !timer.hasExpired());
-
-    return queue.isEmpty() && activeThreads == 0;
+    if (!waitForDone(timer))
+        return false;
+    reset();
+    // New jobs might have started during reset, but return anyway
+    // as the active thread and task count did reach 0 once, and
+    // race conditions are outside our scope.
+    return true;
 }
 
 void QThreadPoolPrivate::clear()
 {
     QMutexLocker locker(&mutex);
-    for (QueuePage *page : qAsConst(queue)) {
+    while (!queue.isEmpty()) {
+        auto *page = queue.takeLast();
         while (!page->isFinished()) {
             QRunnable *r = page->pop();
-            if (r && r->autoDelete() && !--r->ref)
+            if (r && r->autoDelete()) {
+                locker.unlock();
                 delete r;
+                locker.relock();
+            }
         }
+        delete page;
     }
-    qDeleteAll(queue);
-    queue.clear();
 }
 
 /*!
@@ -365,19 +346,15 @@ bool QThreadPool::tryTake(QRunnable *runnable)
 
     if (runnable == nullptr)
         return false;
-    {
-        QMutexLocker locker(&d->mutex);
 
-        for (QueuePage *page : qAsConst(d->queue)) {
-            if (page->tryTake(runnable)) {
-                if (page->isFinished()) {
-                    d->queue.removeOne(page);
-                    delete page;
-                }
-                if (runnable->autoDelete())
-                    --runnable->ref; // undo ++ref in start()
-                return true;
+    QMutexLocker locker(&d->mutex);
+    for (QueuePage *page : std::as_const(d->queue)) {
+        if (page->tryTake(runnable)) {
+            if (page->isFinished()) {
+                d->queue.removeOne(page);
+                delete page;
             }
+            return true;
         }
     }
 
@@ -395,13 +372,13 @@ void QThreadPoolPrivate::stealAndRunRunnable(QRunnable *runnable)
     Q_Q(QThreadPool);
     if (!q->tryTake(runnable))
         return;
-    const bool del = runnable->autoDelete() && !runnable->ref; // tryTake already deref'ed
+    // If autoDelete() is false, runnable might already be deleted after run(), so check status now.
+    const bool del = runnable->autoDelete();
 
     runnable->run();
 
-    if (del) {
+    if (del)
         delete runnable;
-    }
 }
 
 /*!
@@ -413,7 +390,7 @@ void QThreadPoolPrivate::stealAndRunRunnable(QRunnable *runnable)
 
     \ingroup thread
 
-    QThreadPool manages and recyles individual QThread objects to help reduce
+    QThreadPool manages and recycles individual QThread objects to help reduce
     thread creation costs in programs that use threads. Each Qt application
     has one global QThreadPool object, which can be accessed by calling
     globalInstance().
@@ -462,7 +439,14 @@ void QThreadPoolPrivate::stealAndRunRunnable(QRunnable *runnable)
 */
 QThreadPool::QThreadPool(QObject *parent)
     : QObject(*new QThreadPoolPrivate, parent)
-{ }
+{
+    Q_D(QThreadPool);
+    connect(this, &QObject::objectNameChanged, this, [d](const QString &newName) {
+        // We keep a copy of the name under our own lock, so we can access it thread-safely.
+        QMutexLocker locker(&d->mutex);
+        d->objectName = newName;
+    });
+}
 
 /*!
     Destroys the QThreadPool.
@@ -470,7 +454,10 @@ QThreadPool::QThreadPool(QObject *parent)
 */
 QThreadPool::~QThreadPool()
 {
+    Q_D(QThreadPool);
     waitForDone();
+    Q_ASSERT(d->queue.isEmpty());
+    Q_ASSERT(d->allThreads.isEmpty());
 }
 
 /*!
@@ -478,7 +465,28 @@ QThreadPool::~QThreadPool()
 */
 QThreadPool *QThreadPool::globalInstance()
 {
-    return theInstance();
+    Q_CONSTINIT static QPointer<QThreadPool> theInstance;
+    Q_CONSTINIT static QBasicMutex theMutex;
+
+    const QMutexLocker locker(&theMutex);
+    if (theInstance.isNull() && !QCoreApplication::closingDown())
+        theInstance = new QThreadPool();
+    return theInstance;
+}
+
+/*!
+    Returns the QThreadPool instance for Qt Gui.
+    \internal
+*/
+QThreadPool *QThreadPoolPrivate::qtGuiInstance()
+{
+    Q_CONSTINIT static QPointer<QThreadPool> guiInstance;
+    Q_CONSTINIT static QBasicMutex theMutex;
+
+    const QMutexLocker locker(&theMutex);
+    if (guiInstance.isNull() && !QCoreApplication::closingDown())
+        guiInstance = new QThreadPool();
+    return guiInstance;
 }
 
 /*!
@@ -503,12 +511,9 @@ void QThreadPool::start(QRunnable *runnable, int priority)
 
     Q_D(QThreadPool);
     QMutexLocker locker(&d->mutex);
-    if (!d->tryStart(runnable)) {
-        d->enqueueTask(runnable, priority);
 
-        if (!d->waitingThreads.isEmpty())
-            d->waitingThreads.takeFirst()->runnableReady.wakeOne();
-    }
+    if (!d->tryStart(runnable))
+        d->enqueueTask(runnable, priority);
 }
 
 /*!
@@ -550,7 +555,10 @@ bool QThreadPool::tryStart(QRunnable *runnable)
 
     Q_D(QThreadPool);
     QMutexLocker locker(&d->mutex);
-    return d->tryStart(runnable);
+    if (d->tryStart(runnable))
+        return true;
+
+    return false;
 }
 
 /*!
@@ -569,7 +577,7 @@ bool QThreadPool::tryStart(std::function<void()> functionToRun)
 
     Q_D(QThreadPool);
     QMutexLocker locker(&d->mutex);
-    if (!d->allThreads.isEmpty() && d->activeThreadCount() >= d->maxThreadCount)
+    if (!d->allThreads.isEmpty() && d->areAllThreadsActive())
         return false;
 
     QRunnable *runnable = QRunnable::create(std::move(functionToRun));
@@ -580,8 +588,9 @@ bool QThreadPool::tryStart(std::function<void()> functionToRun)
 }
 
 /*! \property QThreadPool::expiryTimeout
+    \brief the thread expiry timeout value in milliseconds.
 
-    Threads that are unused for \a expiryTimeout milliseconds are considered
+    Threads that are unused for \e expiryTimeout milliseconds are considered
     to have expired and will exit. Such threads will be restarted as needed.
     The default \a expiryTimeout is 30000 milliseconds (30 seconds). If
     \a expiryTimeout is negative, newly created threads will not expire, e.g.,
@@ -596,12 +605,14 @@ bool QThreadPool::tryStart(std::function<void()> functionToRun)
 int QThreadPool::expiryTimeout() const
 {
     Q_D(const QThreadPool);
+    QMutexLocker locker(&d->mutex);
     return d->expiryTimeout;
 }
 
 void QThreadPool::setExpiryTimeout(int expiryTimeout)
 {
     Q_D(QThreadPool);
+    QMutexLocker locker(&d->mutex);
     if (d->expiryTimeout == expiryTimeout)
         return;
     d->expiryTimeout = expiryTimeout;
@@ -609,8 +620,9 @@ void QThreadPool::setExpiryTimeout(int expiryTimeout)
 
 /*! \property QThreadPool::maxThreadCount
 
-    This property represents the maximum number of threads used by the thread
-    pool.
+    \brief the maximum number of threads used by the thread pool. This property
+    will default to the value of QThread::idealThreadCount() at the moment the
+    QThreadPool object is created.
 
     \note The thread pool will always use at least 1 thread, even if
     \a maxThreadCount limit is zero or negative.
@@ -621,7 +633,8 @@ void QThreadPool::setExpiryTimeout(int expiryTimeout)
 int QThreadPool::maxThreadCount() const
 {
     Q_D(const QThreadPool);
-    return d->maxThreadCount;
+    QMutexLocker locker(&d->mutex);
+    return d->requestedMaxThreadCount;
 }
 
 void QThreadPool::setMaxThreadCount(int maxThreadCount)
@@ -629,16 +642,16 @@ void QThreadPool::setMaxThreadCount(int maxThreadCount)
     Q_D(QThreadPool);
     QMutexLocker locker(&d->mutex);
 
-    if (maxThreadCount == d->maxThreadCount)
+    if (maxThreadCount == d->requestedMaxThreadCount)
         return;
 
-    d->maxThreadCount = maxThreadCount;
+    d->requestedMaxThreadCount = maxThreadCount;
     d->tryToStartMoreThreads();
 }
 
 /*! \property QThreadPool::activeThreadCount
 
-    This property represents the number of active threads in the thread pool.
+    \brief the number of active threads in the thread pool.
 
     \note It is possible for this function to return a value that is greater
     than maxThreadCount(). See reserveThread() for more details.
@@ -659,7 +672,10 @@ int QThreadPool::activeThreadCount() const
     Once you are done with the thread, call releaseThread() to allow it to be
     reused.
 
-    \note This function will always increase the number of active threads.
+    \note Even if reserving maxThreadCount() threads or more, the thread pool
+    will still allow a minimum of one thread.
+
+    \note This function will increase the reported number of active threads.
     This means that by using this function, it is possible for
     activeThreadCount() to return a value greater than maxThreadCount() .
 
@@ -673,9 +689,7 @@ void QThreadPool::reserveThread()
 }
 
 /*! \property QThreadPool::stackSize
-
-    This property contains the stack size for the thread pool worker
-    threads.
+    \brief the stack size for the thread pool worker threads.
 
     The value of the property is only used when the thread pool creates
     new threads. Changing it has no effect for already created
@@ -689,15 +703,45 @@ void QThreadPool::reserveThread()
 void QThreadPool::setStackSize(uint stackSize)
 {
     Q_D(QThreadPool);
+    QMutexLocker locker(&d->mutex);
     d->stackSize = stackSize;
 }
 
 uint QThreadPool::stackSize() const
 {
     Q_D(const QThreadPool);
+    QMutexLocker locker(&d->mutex);
     return d->stackSize;
 }
 
+/*! \property QThreadPool::threadPriority
+    \brief the thread priority for new worker threads.
+
+    The value of the property is only used when the thread pool starts
+    new threads. Changing it has no effect for already running threads.
+
+    The default value is QThread::InheritPriority, which makes QThread
+    use the same priority as the one the QThreadPool object lives in.
+
+    \sa QThread::Priority
+
+    \since 6.2
+*/
+
+void QThreadPool::setThreadPriority(QThread::Priority priority)
+{
+    Q_D(QThreadPool);
+    QMutexLocker locker(&d->mutex);
+    d->threadPriority = priority;
+}
+
+QThread::Priority QThreadPool::threadPriority() const
+{
+    Q_D(const QThreadPool);
+    QMutexLocker locker(&d->mutex);
+    return d->threadPriority;
+}
+
 /*!
     Releases a thread previously reserved by a call to reserveThread().
 
@@ -718,6 +762,57 @@ void QThreadPool::releaseThread()
     d->tryToStartMoreThreads();
 }
 
+/*!
+    Releases a thread previously reserved with reserveThread() and uses it
+    to run \a runnable.
+
+    Note that the thread pool takes ownership of the \a runnable if
+    \l{QRunnable::autoDelete()}{runnable->autoDelete()} returns \c true,
+    and the \a runnable will be deleted automatically by the thread
+    pool after the \l{QRunnable::run()}{runnable->run()} returns. If
+    \l{QRunnable::autoDelete()}{runnable->autoDelete()} returns \c false,
+    ownership of \a runnable remains with the caller. Note that
+    changing the auto-deletion on \a runnable after calling this
+    functions results in undefined behavior.
+
+    \note Calling this when no threads are reserved results in
+    undefined behavior.
+
+    \since 6.3
+    \sa reserveThread(), start()
+*/
+void QThreadPool::startOnReservedThread(QRunnable *runnable)
+{
+    if (!runnable)
+        return releaseThread();
+
+    Q_D(QThreadPool);
+    QMutexLocker locker(&d->mutex);
+    Q_ASSERT(d->reservedThreads > 0);
+    --d->reservedThreads;
+
+    if (!d->tryStart(runnable)) {
+        // This can only happen if we reserved max threads,
+        // and something took the one minimum thread.
+        d->enqueueTask(runnable, INT_MAX);
+    }
+}
+
+/*!
+    \overload
+    \since 6.3
+
+    Releases a thread previously reserved with reserveThread() and uses it
+    to run \a functionToRun.
+*/
+void QThreadPool::startOnReservedThread(std::function<void()> functionToRun)
+{
+    if (!functionToRun)
+        return releaseThread();
+
+    startOnReservedThread(QRunnable::create(std::move(functionToRun)));
+}
+
 /*!
     Waits up to \a msecs milliseconds for all threads to exit and removes all
     threads from the thread pool. Returns \c true if all threads were removed;
@@ -745,24 +840,22 @@ void QThreadPool::clear()
     d->clear();
 }
 
-#if QT_DEPRECATED_SINCE(5, 9)
 /*!
-    \since 5.5
-    \obsolete use tryTake() instead, but note the different deletion rules.
+    \since 6.0
 
-    Removes the specified \a runnable from the queue if it is not yet started.
-    The runnables for which \l{QRunnable::autoDelete()}{runnable->autoDelete()}
-    returns \c true are deleted.
-
-    \sa start(), tryTake()
+    Returns \c true if \a thread is a thread managed by this thread pool.
 */
-void QThreadPool::cancel(QRunnable *runnable)
+bool QThreadPool::contains(const QThread *thread) const
 {
-    if (tryTake(runnable) && runnable->autoDelete() && !runnable->ref) // tryTake already deref'ed
-        delete runnable;
+    Q_D(const QThreadPool);
+    const QThreadPoolThread *poolThread = qobject_cast<const QThreadPoolThread *>(thread);
+    if (!poolThread)
+        return false;
+    QMutexLocker locker(&d->mutex);
+    return d->allThreads.contains(const_cast<QThreadPoolThread *>(poolThread));
 }
-#endif
 
 QT_END_NAMESPACE
 
 #include "moc_qthreadpool.cpp"
+#include "qthreadpool.moc"
