@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #ifndef  QT_NO_CURSOR
 #include "qwindowscursor.h"
@@ -50,6 +14,8 @@
 #include <QtGui/qscreen.h>
 #include <QtGui/private/qguiapplication_p.h> // getPixmapCursor()
 #include <QtGui/private/qhighdpiscaling_p.h>
+#include <QtGui/private/qpixmap_win_p.h>
+#include <QtCore/private/qwinregistry_p.h>
 
 #include <QtCore/qdebug.h>
 #include <QtCore/qscopedpointer.h>
@@ -64,25 +30,21 @@ static bool initResources()
 
 QT_BEGIN_NAMESPACE
 
-Q_GUI_EXPORT HBITMAP qt_pixmapToWinHBITMAP(const QPixmap &p, int hbitmapFormat = 0);
-Q_GUI_EXPORT HBITMAP qt_createIconMask(const QBitmap &bitmap);
-
 /*!
     \class QWindowsCursorCacheKey
     \brief Cache key for storing values in a QHash with a QCursor as key.
 
     \internal
-    \ingroup qt-lighthouse-win
 */
 
 QWindowsPixmapCursorCacheKey::QWindowsPixmapCursorCacheKey(const QCursor &c)
     : bitmapCacheKey(c.pixmap().cacheKey()), maskCacheKey(0)
 {
     if (!bitmapCacheKey) {
-        Q_ASSERT(c.bitmap());
-        Q_ASSERT(c.mask());
-        bitmapCacheKey = c.bitmap()->cacheKey();
-        maskCacheKey = c.mask()->cacheKey();
+        Q_ASSERT(!c.bitmap().isNull());
+        Q_ASSERT(!c.mask().isNull());
+        bitmapCacheKey = c.bitmap().cacheKey();
+        maskCacheKey = c.mask().cacheKey();
     }
 }
 
@@ -96,14 +58,13 @@ QWindowsPixmapCursorCacheKey::QWindowsPixmapCursorCacheKey(const QCursor &c)
     as do the Window manager frames (resize/move handles).
 
     \internal
-    \ingroup qt-lighthouse-win
     \sa QWindowsWindowCursor
 */
 
 HCURSOR QWindowsCursor::createPixmapCursor(QPixmap pixmap, const QPoint &hotSpot, qreal scaleFactor)
 {
     HCURSOR cur = nullptr;
-    const qreal pixmapScaleFactor = scaleFactor / pixmap.devicePixelRatioF();
+    const qreal pixmapScaleFactor = scaleFactor / pixmap.devicePixelRatio();
     if (!qFuzzyCompare(pixmapScaleFactor, 1)) {
         pixmap = pixmap.scaled((pixmapScaleFactor * QSizeF(pixmap.size())).toSize(),
                                Qt::KeepAspectRatio, Qt::SmoothTransformation);
@@ -168,10 +129,10 @@ static HCURSOR createBitmapCursor(const QImage &bbits, const QImage &mbits,
 // Create a cursor from image and mask of the format QImage::Format_Mono.
 static HCURSOR createBitmapCursor(const QCursor &cursor, qreal scaleFactor = 1)
 {
-    Q_ASSERT(cursor.shape() == Qt::BitmapCursor && cursor.bitmap());
-    QImage bbits = cursor.bitmap()->toImage();
-    QImage mbits = cursor.mask()->toImage();
-    scaleFactor /= bbits.devicePixelRatioF();
+    Q_ASSERT(cursor.shape() == Qt::BitmapCursor && !cursor.bitmap().isNull());
+    QImage bbits = cursor.bitmap().toImage();
+    QImage mbits = cursor.mask().toImage();
+    scaleFactor /= bbits.devicePixelRatio();
     if (!qFuzzyCompare(scaleFactor, 1)) {
         const QSize scaledSize = (QSizeF(bbits.size()) * scaleFactor).toSize();
         bbits = bbits.scaled(scaledSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
@@ -589,7 +550,7 @@ QWindowsCursor::QWindowsCursor(const QPlatformScreen *screen)
     : m_screen(screen)
 {
     static const bool dummy = initResources();
-    Q_UNUSED(dummy)
+    Q_UNUSED(dummy);
 }
 
 inline CursorHandlePtr QWindowsCursor::cursorHandle(const QCursor &cursor)
@@ -653,6 +614,11 @@ void QWindowsCursor::clearOverrideCursor()
         SetCursor(m_overriddenCursor);
         m_overriddenCursor = m_overrideCursor = nullptr;
     }
+    auto &windows = QWindowsContext::instance()->windows();
+    for (auto it = windows.cbegin(), end = windows.cend(); it != end; ++it) {
+        if (it.value()->screen() == m_screen)
+            it.value()->setFlag(QWindowsWindow::RestoreOverrideCursor);
+    }
 }
 
 QPoint QWindowsCursor::mousePosition()
@@ -686,6 +652,30 @@ void QWindowsCursor::setPos(const QPoint &pos)
     SetCursorPos(pos.x() , pos.y());
 }
 
+/*
+    The standard size is 32x32, even though the cursor is actually just
+    16 pixels large. If a large cursor is set in the accessibility settings,
+    then the cursor increases with 8 pixels for each step.
+*/
+QSize QWindowsCursor::size() const
+{
+    const QPair<DWORD,bool> cursorSizeSetting =
+        QWinRegistryKey(HKEY_CURRENT_USER, LR"(Control Panel\Cursors)")
+                       .dwordValue(L"CursorBaseSize");
+    const int baseSize = screenCursorSize(m_screen).width() / 2;
+    if (!cursorSizeSetting.second)
+        return QSize(baseSize / 2, baseSize / 2);
+
+    // The registry values are dpi-independent, so we need to scale the result.
+    int cursorSizeValue = cursorSizeSetting.first * m_screen->logicalDpi().first
+                                                  / m_screen->logicalBaseDpi().first;
+
+    // map from registry value 32-256 to 0-14, and from there to pixels
+    cursorSizeValue = (cursorSizeValue - 2 * baseSize) / baseSize;
+    const int cursorSize = baseSize + cursorSizeValue * (baseSize / 2);
+    return QSize(cursorSize, cursorSize);
+}
+
 QPixmap QWindowsCursor::dragDefaultCursor(Qt::DropAction action) const
 {
     switch (action) {
@@ -791,7 +781,6 @@ HCURSOR QWindowsCursor::hCursor(const QCursor &c) const
      cursor handle resource.
 
     \internal
-    \ingroup qt-lighthouse-win
     \sa QWindowsCursor
 */
 
