@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qxcbintegration.h"
 #include "qxcbconnection.h"
@@ -46,31 +10,37 @@
 #include "qxcbbackingstore.h"
 #include "qxcbnativeinterface.h"
 #include "qxcbclipboard.h"
+#include "qxcbeventqueue.h"
+#include "qxcbeventdispatcher.h"
+#if QT_CONFIG(draganddrop)
 #include "qxcbdrag.h"
+#endif
 #include "qxcbglintegration.h"
 
 #ifndef QT_NO_SESSIONMANAGER
 #include "qxcbsessionmanager.h"
 #endif
+#include "qxcbxsettings.h"
 
 #include <xcb/xcb.h>
 
-#include <QtEventDispatcherSupport/private/qgenericunixeventdispatcher_p.h>
-#include <QtFontDatabaseSupport/private/qgenericunixfontdatabase_p.h>
-#include <QtServiceSupport/private/qgenericunixservices_p.h>
+#include <QtGui/private/qgenericunixfontdatabase_p.h>
+#include <QtGui/private/qgenericunixservices_p.h>
 
 #include <stdio.h>
 
 #include <QtGui/private/qguiapplication_p.h>
 
 #if QT_CONFIG(xcb_xlib)
+#define register        /* C++17 deprecated register */
 #include <X11/Xlib.h>
+#undef register
+#endif
 #if QT_CONFIG(xcb_native_painting)
 #include "qxcbnativepainting.h"
 #include "qpixmap_x11_p.h"
 #include "qbackingstore_x11_p.h"
 #endif
-#endif
 
 #include <qpa/qplatforminputcontextfactory_p.h>
 #include <private/qgenericunixthemes_p.h>
@@ -79,10 +49,10 @@
 #include <QtGui/QOpenGLContext>
 #include <QtGui/QScreen>
 #include <QtGui/QOffscreenSurface>
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
 #include <qpa/qplatformaccessibility.h>
-#ifndef QT_NO_ACCESSIBILITY_ATSPI_BRIDGE
-#include <QtLinuxAccessibilitySupport/private/bridge_p.h>
+#if QT_CONFIG(accessibility_atspi_bridge)
+#include <QtGui/private/qspiaccessiblebridge_p.h>
 #endif
 #endif
 
@@ -95,16 +65,18 @@
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 // Find out if our parent process is gdb by looking at the 'exe' symlink under /proc,.
 // or, for older Linuxes, read out 'cmdline'.
 static bool runningUnderDebugger()
 {
 #if defined(QT_DEBUG) && defined(Q_OS_LINUX)
-    const QString parentProc = QLatin1String("/proc/") + QString::number(getppid());
-    const QFileInfo parentProcExe(parentProc + QLatin1String("/exe"));
+    const QString parentProc = "/proc/"_L1 + QString::number(getppid());
+    const QFileInfo parentProcExe(parentProc + "/exe"_L1);
     if (parentProcExe.isSymLink())
-        return parentProcExe.symLinkTarget().endsWith(QLatin1String("/gdb"));
-    QFile f(parentProc + QLatin1String("/cmdline"));
+        return parentProcExe.symLinkTarget().endsWith("/gdb"_L1);
+    QFile f(parentProc + "/cmdline"_L1);
     if (!f.open(QIODevice::ReadOnly))
         return false;
     QByteArray s;
@@ -121,14 +93,16 @@ static bool runningUnderDebugger()
 #endif
 }
 
-QXcbIntegration *QXcbIntegration::m_instance = Q_NULLPTR;
+QXcbIntegration *QXcbIntegration::m_instance = nullptr;
 
 QXcbIntegration::QXcbIntegration(const QStringList &parameters, int &argc, char **argv)
     : m_services(new QGenericUnixServices)
-    , m_instanceName(0)
+    , m_instanceName(nullptr)
     , m_canGrab(true)
     , m_defaultVisualId(UINT_MAX)
 {
+    Q_UNUSED(parameters);
+
     m_instance = this;
     qApp->setAttribute(Qt::AA_CompressHighFrequencyEvents, true);
 
@@ -139,7 +113,7 @@ QXcbIntegration::QXcbIntegration(const QStringList &parameters, int &argc, char
     m_nativeInterface.reset(new QXcbNativeInterface);
 
     // Parse arguments
-    const char *displayName = 0;
+    const char *displayName = nullptr;
     bool noGrabArg = false;
     bool doGrabArg = false;
     if (argc) {
@@ -170,7 +144,7 @@ QXcbIntegration::QXcbIntegration(const QStringList &parameters, int &argc, char
 
     bool underDebugger = runningUnderDebugger();
     if (noGrabArg && doGrabArg && underDebugger) {
-        qWarning("Both -nograb and -dograb command line arguments specified. Please pick one. -nograb takes prcedence");
+        qWarning("Both -nograb and -dograb command line arguments specified. Please pick one. -nograb takes precedence");
         doGrabArg = false;
     }
 
@@ -186,27 +160,11 @@ QXcbIntegration::QXcbIntegration(const QStringList &parameters, int &argc, char
     if (canNotGrabEnv)
         m_canGrab = false;
 
-    const int numParameters = parameters.size();
-    m_connections.reserve(1 + numParameters / 2);
-    auto conn = new QXcbConnection(m_nativeInterface.data(), m_canGrab, m_defaultVisualId, displayName);
-    if (conn->isConnected())
-        m_connections << conn;
-    else
-        delete conn;
-
-    for (int i = 0; i < numParameters - 1; i += 2) {
-        qCDebug(lcQpaScreen) << "connecting to additional display: " << parameters.at(i) << parameters.at(i+1);
-        QString display = parameters.at(i) + QLatin1Char(':') + parameters.at(i+1);
-        conn = new QXcbConnection(m_nativeInterface.data(), m_canGrab, m_defaultVisualId, display.toLatin1().constData());
-        if (conn->isConnected())
-            m_connections << conn;
-        else
-            delete conn;
-    }
-
-    if (m_connections.isEmpty()) {
-        qCritical("Could not connect to any X display.");
-        exit(1);
+    m_connection = new QXcbConnection(m_nativeInterface.data(), m_canGrab, m_defaultVisualId, displayName);
+    if (!m_connection->isConnected()) {
+        delete m_connection;
+        m_connection = nullptr;
+        return;
     }
 
     m_fontDatabase.reset(new QGenericUnixFontDatabase());
@@ -214,15 +172,16 @@ QXcbIntegration::QXcbIntegration(const QStringList &parameters, int &argc, char
 #if QT_CONFIG(xcb_native_painting)
     if (nativePaintingEnabled()) {
         qCDebug(lcQpaXcb, "QXCB USING NATIVE PAINTING");
-        qt_xcb_native_x11_info_init(defaultConnection());
+        qt_xcb_native_x11_info_init(connection());
     }
 #endif
 }
 
 QXcbIntegration::~QXcbIntegration()
 {
-    qDeleteAll(m_connections);
-    m_instance = Q_NULLPTR;
+    delete m_connection;
+    m_connection = nullptr;
+    m_instance = nullptr;
 }
 
 QPlatformPixmap *QXcbIntegration::createPlatformPixmap(QPlatformPixmap::PixelType type) const
@@ -237,10 +196,11 @@ QPlatformPixmap *QXcbIntegration::createPlatformPixmap(QPlatformPixmap::PixelTyp
 
 QPlatformWindow *QXcbIntegration::createPlatformWindow(QWindow *window) const
 {
-    QXcbScreen *screen = static_cast<QXcbScreen *>(window->screen()->handle());
-    QXcbGlIntegration *glIntegration = screen->connection()->glIntegration();
-    if (window->type() != Qt::Desktop) {
+    QXcbGlIntegration *glIntegration = nullptr;
+    const bool isTrayIconWindow = QXcbWindow::isTrayIconWindow(window);;
+    if (window->type() != Qt::Desktop && !isTrayIconWindow) {
         if (window->supportsOpenGL()) {
+            glIntegration = connection()->glIntegration();
             if (glIntegration) {
                 QXcbWindow *xcbWindow = glIntegration->createWindow(window);
                 xcbWindow->create();
@@ -255,7 +215,7 @@ QPlatformWindow *QXcbIntegration::createPlatformWindow(QWindow *window) const
         }
     }
 
-    Q_ASSERT(window->type() == Qt::Desktop || !window->supportsOpenGL()
+    Q_ASSERT(window->type() == Qt::Desktop || isTrayIconWindow || !window->supportsOpenGL()
              || (!glIntegration && window->surfaceType() == QSurface::RasterGLSurface)); // for VNC
     QXcbWindow *xcbWindow = new QXcbWindow(window);
     xcbWindow->create();
@@ -270,24 +230,54 @@ QPlatformWindow *QXcbIntegration::createForeignWindow(QWindow *window, WId nativ
 #ifndef QT_NO_OPENGL
 QPlatformOpenGLContext *QXcbIntegration::createPlatformOpenGLContext(QOpenGLContext *context) const
 {
-    QXcbScreen *screen = static_cast<QXcbScreen *>(context->screen()->handle());
-    QXcbGlIntegration *glIntegration = screen->connection()->glIntegration();
+    QXcbGlIntegration *glIntegration = m_connection->glIntegration();
     if (!glIntegration) {
         qWarning("QXcbIntegration: Cannot create platform OpenGL context, neither GLX nor EGL are enabled");
-        return Q_NULLPTR;
+        return nullptr;
     }
     return glIntegration->createPlatformOpenGLContext(context);
 }
+
+# if QT_CONFIG(xcb_glx_plugin)
+QOpenGLContext *QXcbIntegration::createOpenGLContext(GLXContext context, void *visualInfo, QOpenGLContext *shareContext) const
+{
+    using namespace QNativeInterface::Private;
+    if (auto *glxIntegration = dynamic_cast<QGLXIntegration*>(m_connection->glIntegration()))
+        return glxIntegration->createOpenGLContext(context, visualInfo, shareContext);
+    else
+        return nullptr;
+}
+# endif
+
+#if QT_CONFIG(egl)
+QOpenGLContext *QXcbIntegration::createOpenGLContext(EGLContext context, EGLDisplay display, QOpenGLContext *shareContext) const
+{
+    using namespace QNativeInterface::Private;
+    if (auto *eglIntegration = dynamic_cast<QEGLIntegration*>(m_connection->glIntegration()))
+        return eglIntegration->createOpenGLContext(context, display, shareContext);
+    else
+        return nullptr;
+}
 #endif
 
+#endif // QT_NO_OPENGL
+
 QPlatformBackingStore *QXcbIntegration::createPlatformBackingStore(QWindow *window) const
 {
-#if QT_CONFIG(xcb_native_painting)
-    if (nativePaintingEnabled())
-        return new QXcbNativeBackingStore(window);
-#endif
+    QPlatformBackingStore *backingStore = nullptr;
 
-    return new QXcbBackingStore(window);
+    const bool isTrayIconWindow = QXcbWindow::isTrayIconWindow(window);
+    if (isTrayIconWindow) {
+        backingStore = new QXcbSystemTrayBackingStore(window);
+#if QT_CONFIG(xcb_native_painting)
+    } else if (nativePaintingEnabled()) {
+        backingStore = new QXcbNativeBackingStore(window);
+#endif
+    } else {
+        backingStore = new QXcbBackingStore(window);
+    }
+    Q_ASSERT(backingStore);
+    return backingStore;
 }
 
 QPlatformOffscreenSurface *QXcbIntegration::createPlatformOffscreenSurface(QOffscreenSurface *surface) const
@@ -296,7 +286,7 @@ QPlatformOffscreenSurface *QXcbIntegration::createPlatformOffscreenSurface(QOffs
     QXcbGlIntegration *glIntegration = screen->connection()->glIntegration();
     if (!glIntegration) {
         qWarning("QXcbIntegration: Cannot create platform offscreen surface, neither GLX nor EGL are enabled");
-        return Q_NULLPTR;
+        return nullptr;
     }
     return glIntegration->createPlatformOffscreenSurface(surface);
 }
@@ -307,10 +297,8 @@ bool QXcbIntegration::hasCapability(QPlatformIntegration::Capability cap) const
     case OpenGL:
     case ThreadedOpenGL:
     {
-        const auto *connection = qAsConst(m_connections).first();
-        if (const auto *integration = connection->glIntegration())
-            return cap != ThreadedOpenGL
-                || (connection->threadedEventHandling() && integration->supportsThreadedOpenGL());
+        if (const auto *integration = connection()->glIntegration())
+            return cap != ThreadedOpenGL || integration->supportsThreadedOpenGL();
         return false;
     }
 
@@ -324,8 +312,8 @@ bool QXcbIntegration::hasCapability(QPlatformIntegration::Capability cap) const
 
     case SwitchableWidgetComposition:
     {
-        return m_connections.at(0)->glIntegration()
-            && m_connections.at(0)->glIntegration()->supportsSwitchableWidgetComposition();
+        return m_connection->glIntegration()
+            && m_connection->glIntegration()->supportsSwitchableWidgetComposition();
     }
 
     default: return QPlatformIntegration::hasCapability(cap);
@@ -334,23 +322,40 @@ bool QXcbIntegration::hasCapability(QPlatformIntegration::Capability cap) const
 
 QAbstractEventDispatcher *QXcbIntegration::createEventDispatcher() const
 {
-    QAbstractEventDispatcher *dispatcher = createUnixEventDispatcher();
-    for (int i = 0; i < m_connections.size(); i++)
-        m_connections[i]->eventReader()->registerEventDispatcher(dispatcher);
-    return dispatcher;
+    return QXcbEventDispatcher::createEventDispatcher(connection());
 }
 
+using namespace Qt::Literals::StringLiterals;
+static const auto xsNetCursorBlink = "Net/CursorBlink"_ba;
+static const auto xsNetCursorBlinkTime = "Net/CursorBlinkTime"_ba;
+static const auto xsNetDoubleClickTime = "Net/DoubleClickTime"_ba;
+static const auto xsNetDoubleClickDistance = "Net/DoubleClickDistance"_ba;
+static const auto xsNetDndDragThreshold = "Net/DndDragThreshold"_ba;
+
 void QXcbIntegration::initialize()
 {
-    const QLatin1String defaultInputContext("compose");
+    const auto defaultInputContext = "compose"_L1;
     // Perform everything that may potentially need the event dispatcher (timers, socket
     // notifiers) here instead of the constructor.
     QString icStr = QPlatformInputContextFactory::requested();
     if (icStr.isNull())
         icStr = defaultInputContext;
     m_inputContext.reset(QPlatformInputContextFactory::create(icStr));
-    if (!m_inputContext && icStr != defaultInputContext && icStr != QLatin1String("none"))
+    if (!m_inputContext && icStr != defaultInputContext && icStr != "none"_L1)
         m_inputContext.reset(QPlatformInputContextFactory::create(defaultInputContext));
+
+    connection()->keyboard()->initialize();
+
+    auto notifyThemeChanged = [](QXcbVirtualDesktop *, const QByteArray &, const QVariant &, void *) {
+        QWindowSystemInterface::handleThemeChange();
+    };
+
+    auto *xsettings = connection()->primaryScreen()->xSettings();
+    xsettings->registerCallbackForProperty(xsNetCursorBlink, notifyThemeChanged, this);
+    xsettings->registerCallbackForProperty(xsNetCursorBlinkTime, notifyThemeChanged, this);
+    xsettings->registerCallbackForProperty(xsNetDoubleClickTime, notifyThemeChanged, this);
+    xsettings->registerCallbackForProperty(xsNetDoubleClickDistance, notifyThemeChanged, this);
+    xsettings->registerCallbackForProperty(xsNetDndDragThreshold, notifyThemeChanged, this);
 }
 
 void QXcbIntegration::moveToScreen(QWindow *window, int screen)
@@ -372,14 +377,23 @@ QPlatformNativeInterface * QXcbIntegration::nativeInterface() const
 #ifndef QT_NO_CLIPBOARD
 QPlatformClipboard *QXcbIntegration::clipboard() const
 {
-    return m_connections.at(0)->clipboard();
+    return m_connection->clipboard();
 }
 #endif
 
-#ifndef QT_NO_DRAGANDDROP
+#if QT_CONFIG(draganddrop)
+#include <private/qsimpledrag_p.h>
 QPlatformDrag *QXcbIntegration::drag() const
 {
-    return m_connections.at(0)->drag();
+    static const bool useSimpleDrag = qEnvironmentVariableIsSet("QT_XCB_USE_SIMPLE_DRAG");
+    if (Q_UNLIKELY(useSimpleDrag)) { // This is useful for testing purposes
+        static QSimpleDrag *simpleDrag = nullptr;
+        if (!simpleDrag)
+            simpleDrag = new QSimpleDrag();
+        return simpleDrag;
+    }
+
+    return m_connection->drag();
 }
 #endif
 
@@ -388,7 +402,7 @@ QPlatformInputContext *QXcbIntegration::inputContext() const
     return m_inputContext.data();
 }
 
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
 QPlatformAccessibility *QXcbIntegration::accessibility() const
 {
 #if !defined(QT_NO_ACCESSIBILITY_ATSPI_BRIDGE)
@@ -410,15 +424,12 @@ QPlatformServices *QXcbIntegration::services() const
 
 Qt::KeyboardModifiers QXcbIntegration::queryKeyboardModifiers() const
 {
-    int keybMask = 0;
-    QXcbConnection *conn = m_connections.at(0);
-    QXcbCursor::queryPointer(conn, 0, 0, &keybMask);
-    return conn->keyboard()->translateModifiers(keybMask);
+    return m_connection->queryKeyboardModifiers();
 }
 
 QList<int> QXcbIntegration::possibleKeys(const QKeyEvent *e) const
 {
-    return m_connections.at(0)->keyboard()->possibleKeys(e);
+    return m_connection->keyboard()->possibleKeys(e);
 }
 
 QStringList QXcbIntegration::themeNames() const
@@ -431,12 +442,30 @@ QPlatformTheme *QXcbIntegration::createPlatformTheme(const QString &name) const
     return QGenericUnixTheme::createUnixTheme(name);
 }
 
+#define RETURN_VALID_XSETTINGS(key) { \
+    auto value = connection()->primaryScreen()->xSettings()->setting(key); \
+    if (value.isValid()) return value; \
+}
+
 QVariant QXcbIntegration::styleHint(QPlatformIntegration::StyleHint hint) const
 {
     switch (hint) {
-    case QPlatformIntegration::CursorFlashTime:
-    case QPlatformIntegration::KeyboardInputInterval:
+    case QPlatformIntegration::CursorFlashTime: {
+        bool ok = false;
+        // If cursor blinking is off, returns 0 to keep the cursor awlays display.
+        if (connection()->primaryScreen()->xSettings()->setting(xsNetCursorBlink).toInt(&ok) == 0 && ok)
+            return 0;
+
+        RETURN_VALID_XSETTINGS(xsNetCursorBlinkTime);
+        break;
+    }
     case QPlatformIntegration::MouseDoubleClickInterval:
+        RETURN_VALID_XSETTINGS(xsNetDoubleClickTime);
+        break;
+    case QPlatformIntegration::MouseDoubleClickDistance:
+        RETURN_VALID_XSETTINGS(xsNetDoubleClickDistance);
+        break;
+    case QPlatformIntegration::KeyboardInputInterval:
     case QPlatformIntegration::StartDragTime:
     case QPlatformIntegration::KeyboardAutoRepeatRate:
     case QPlatformIntegration::PasswordMaskDelay:
@@ -446,10 +475,12 @@ QVariant QXcbIntegration::styleHint(QPlatformIntegration::StyleHint hint) const
         // TODO using various xcb, gnome or KDE settings
         break; // Not implemented, use defaults
     case QPlatformIntegration::StartDragDistance: {
+        RETURN_VALID_XSETTINGS(xsNetDndDragThreshold);
+
         // The default (in QPlatformTheme::defaultThemeHint) is 10 pixels, but
         // on a high-resolution screen it makes sense to increase it.
         qreal dpi = 100.0;
-        if (const QXcbScreen *screen = defaultConnection()->primaryScreen()) {
+        if (const QXcbScreen *screen = connection()->primaryScreen()) {
             if (screen->logicalDpi().first > dpi)
                 dpi = screen->logicalDpi().first;
             if (screen->logicalDpi().second > dpi)
@@ -475,7 +506,7 @@ static QString argv0BaseName()
     const QStringList arguments = QCoreApplication::arguments();
     if (!arguments.isEmpty() && !arguments.front().isEmpty()) {
         result = arguments.front();
-        const int lastSlashPos = result.lastIndexOf(QLatin1Char('/'));
+        const int lastSlashPos = result.lastIndexOf(u'/');
         if (lastSlashPos != -1)
             result.remove(0, lastSlashPos + 1);
     }
@@ -520,9 +551,7 @@ QPlatformSessionManager *QXcbIntegration::createPlatformSessionManager(const QSt
 
 void QXcbIntegration::sync()
 {
-    for (int i = 0; i < m_connections.size(); i++) {
-        m_connections.at(i)->sync();
-    }
+    m_connection->sync();
 }
 
 // For QApplication::beep()
@@ -536,6 +565,7 @@ void QXcbIntegration::beep() const
         return;
     xcb_connection_t *connection = static_cast<QXcbScreen *>(screen)->xcb_connection();
     xcb_bell(connection, 0);
+    xcb_flush(connection);
 }
 
 bool QXcbIntegration::nativePaintingEnabled() const
